diff --git a/src/bin/scripts/clusterdb.c b/src/bin/scripts/clusterdb.c
index 8c0e7cf..2f15c82 100644
--- a/src/bin/scripts/clusterdb.c
+++ b/src/bin/scripts/clusterdb.c
@@ -203,7 +203,7 @@ cluster_one_database(const char *dbname, bool verbose, const char *table,
 		appendPQExpBuffer(&sql, " %s", table);
 	appendPQExpBufferChar(&sql, ';');
 
-	conn = connectDatabase(dbname, host, port, username, prompt_password,
+	conn = connectDatabase(dbname, host, port, username, NULL, prompt_password,
 						   progname, false);
 	if (!executeMaintenanceCommand(conn, sql.data, echo))
 	{
diff --git a/src/bin/scripts/common.c b/src/bin/scripts/common.c
index 0deadec..321fc10 100644
--- a/src/bin/scripts/common.c
+++ b/src/bin/scripts/common.c
@@ -53,18 +53,21 @@ handle_help_version_opts(int argc, char *argv[],
 
 /*
  * Make a database connection with the given parameters.  An
- * interactive password prompt is automatically issued if required.
+ * interactive password prompt is automatically issued if required
+ * depending on if a user password is defined directly by the caller
+ * of this routine via password_user.
  */
 PGconn *
 connectDatabase(const char *dbname, const char *pghost, const char *pgport,
-				const char *pguser, enum trivalue prompt_password,
-				const char *progname, bool fail_ok)
+				const char *pguser, const char *password_user,
+				enum trivalue prompt_password, const char *progname,
+				bool fail_ok)
 {
 	PGconn	   *conn;
-	char	   *password = NULL;
+	char	   *password = password_user ? pg_strdup(password_user) : NULL;
 	bool		new_pass;
 
-	if (prompt_password == TRI_YES)
+	if (prompt_password == TRI_YES && password == NULL)
 		password = simple_prompt("Password: ", 100, false);
 
 	/*
@@ -148,14 +151,14 @@ connectMaintenanceDatabase(const char *maintenance_db, const char *pghost,
 
 	/* If a maintenance database name was specified, just connect to it. */
 	if (maintenance_db)
-		return connectDatabase(maintenance_db, pghost, pgport, pguser,
+		return connectDatabase(maintenance_db, pghost, pgport, pguser, NULL,
 							   prompt_password, progname, false);
 
 	/* Otherwise, try postgres first and then template1. */
-	conn = connectDatabase("postgres", pghost, pgport, pguser, prompt_password,
-						   progname, true);
+	conn = connectDatabase("postgres", pghost, pgport, pguser, NULL,
+						   prompt_password, progname, true);
 	if (!conn)
-		conn = connectDatabase("template1", pghost, pgport, pguser,
+		conn = connectDatabase("template1", pghost, pgport, pguser, NULL,
 							   prompt_password, progname, false);
 
 	return conn;
diff --git a/src/bin/scripts/common.h b/src/bin/scripts/common.h
index b5ce1ed..13d9d8c 100644
--- a/src/bin/scripts/common.h
+++ b/src/bin/scripts/common.h
@@ -31,8 +31,8 @@ extern void handle_help_version_opts(int argc, char *argv[],
 
 extern PGconn *connectDatabase(const char *dbname, const char *pghost,
 				const char *pgport, const char *pguser,
-				enum trivalue prompt_password, const char *progname,
-				bool fail_ok);
+				const char *password_user, enum trivalue prompt_password,
+				const char *progname, bool fail_ok);
 
 extern PGconn *connectMaintenanceDatabase(const char *maintenance_db,
 				  const char *pghost, const char *pgport, const char *pguser,
diff --git a/src/bin/scripts/createlang.c b/src/bin/scripts/createlang.c
index 228215c..38cc20d 100644
--- a/src/bin/scripts/createlang.c
+++ b/src/bin/scripts/createlang.c
@@ -140,8 +140,8 @@ main(int argc, char *argv[])
 		printQueryOpt popt;
 		static const bool translate_columns[] = {false, true};
 
-		conn = connectDatabase(dbname, host, port, username, prompt_password,
-							   progname, false);
+		conn = connectDatabase(dbname, host, port, username, NULL,
+							   prompt_password, progname, false);
 
 		printfPQExpBuffer(&sql, "SELECT lanname as \"%s\", "
 				"(CASE WHEN lanpltrusted THEN '%s' ELSE '%s' END) as \"%s\" "
@@ -180,8 +180,8 @@ main(int argc, char *argv[])
 		if (*p >= 'A' && *p <= 'Z')
 			*p += ('a' - 'A');
 
-	conn = connectDatabase(dbname, host, port, username, prompt_password,
-						   progname, false);
+	conn = connectDatabase(dbname, host, port, username, NULL,
+						   prompt_password, progname, false);
 
 	/*
 	 * Make sure the language isn't already installed
diff --git a/src/bin/scripts/createuser.c b/src/bin/scripts/createuser.c
index c8bcf0d..9e7f84d 100644
--- a/src/bin/scripts/createuser.c
+++ b/src/bin/scripts/createuser.c
@@ -250,8 +250,8 @@ main(int argc, char *argv[])
 	if (login == 0)
 		login = TRI_YES;
 
-	conn = connectDatabase("postgres", host, port, username, prompt_password,
-						   progname, false);
+	conn = connectDatabase("postgres", host, port, username, NULL,
+						   prompt_password, progname, false);
 
 	initPQExpBuffer(&sql);
 
diff --git a/src/bin/scripts/droplang.c b/src/bin/scripts/droplang.c
index 7467328..502ec64 100644
--- a/src/bin/scripts/droplang.c
+++ b/src/bin/scripts/droplang.c
@@ -139,8 +139,8 @@ main(int argc, char *argv[])
 		printQueryOpt popt;
 		static const bool translate_columns[] = {false, true};
 
-		conn = connectDatabase(dbname, host, port, username, prompt_password,
-							   progname, false);
+		conn = connectDatabase(dbname, host, port, username, NULL,
+							   prompt_password, progname, false);
 
 		printfPQExpBuffer(&sql, "SELECT lanname as \"%s\", "
 				"(CASE WHEN lanpltrusted THEN '%s' ELSE '%s' END) as \"%s\" "
@@ -181,8 +181,8 @@ main(int argc, char *argv[])
 		if (*p >= 'A' && *p <= 'Z')
 			*p += ('a' - 'A');
 
-	conn = connectDatabase(dbname, host, port, username, prompt_password,
-						   progname, false);
+	conn = connectDatabase(dbname, host, port, username, NULL,
+						   prompt_password, progname, false);
 
 	/*
 	 * Force schema search path to be just pg_catalog, so that we don't have
diff --git a/src/bin/scripts/dropuser.c b/src/bin/scripts/dropuser.c
index 9160566..10d0691 100644
--- a/src/bin/scripts/dropuser.c
+++ b/src/bin/scripts/dropuser.c
@@ -128,8 +128,8 @@ main(int argc, char *argv[])
 	appendPQExpBuffer(&sql, "DROP ROLE %s%s;",
 					  (if_exists ? "IF EXISTS " : ""), fmtId(dropuser));
 
-	conn = connectDatabase("postgres", host, port, username, prompt_password,
-						   progname, false);
+	conn = connectDatabase("postgres", host, port, username, NULL,
+						   prompt_password, progname, false);
 
 	if (echo)
 		printf("%s\n", sql.data);
diff --git a/src/bin/scripts/reindexdb.c b/src/bin/scripts/reindexdb.c
index decb753..fbf436c 100644
--- a/src/bin/scripts/reindexdb.c
+++ b/src/bin/scripts/reindexdb.c
@@ -297,8 +297,8 @@ reindex_one_database(const char *name, const char *dbname, const char *type,
 		appendPQExpBuffer(&sql, " DATABASE %s", fmtId(name));
 	appendPQExpBufferChar(&sql, ';');
 
-	conn = connectDatabase(dbname, host, port, username, prompt_password,
-						   progname, false);
+	conn = connectDatabase(dbname, host, port, username, NULL,
+						   prompt_password, progname, false);
 
 	if (!executeMaintenanceCommand(conn, sql.data, echo))
 	{
@@ -372,8 +372,8 @@ reindex_system_catalogs(const char *dbname, const char *host, const char *port,
 
 	appendPQExpBuffer(&sql, " SYSTEM %s;", dbname);
 
-	conn = connectDatabase(dbname, host, port, username, prompt_password,
-						   progname, false);
+	conn = connectDatabase(dbname, host, port, username, NULL,
+						   prompt_password, progname, false);
 	if (!executeMaintenanceCommand(conn, sql.data, echo))
 	{
 		fprintf(stderr, _("%s: reindexing of system catalogs failed: %s"),
diff --git a/src/bin/scripts/vacuumdb.c b/src/bin/scripts/vacuumdb.c
index 4ce27b7..8c8d105 100644
--- a/src/bin/scripts/vacuumdb.c
+++ b/src/bin/scripts/vacuumdb.c
@@ -41,7 +41,8 @@ static void vacuum_one_database(const char *dbname, vacuumingOptions *vacopts,
 					int stage,
 					SimpleStringList *tables,
 					const char *host, const char *port,
-					const char *username, enum trivalue prompt_password,
+					const char *username, const char *password,
+					enum trivalue prompt_password,
 					int concurrentCons,
 					const char *progname, bool echo, bool quiet);
 
@@ -294,7 +295,8 @@ main(int argc, char *argv[])
 				vacuum_one_database(dbname, &vacopts,
 									stage,
 									&tables,
-									host, port, username, prompt_password,
+									host, port, username, NULL,
+									prompt_password,
 									concurrentCons,
 									progname, echo, quiet);
 			}
@@ -303,7 +305,8 @@ main(int argc, char *argv[])
 			vacuum_one_database(dbname, &vacopts,
 								ANALYZE_NO_STAGE,
 								&tables,
-								host, port, username, prompt_password,
+								host, port, username, NULL,
+								prompt_password,
 								concurrentCons,
 								progname, echo, quiet);
 	}
@@ -323,13 +326,18 @@ main(int argc, char *argv[])
  * If concurrentCons is > 1, multiple connections are used to vacuum tables
  * in parallel.  In this case and if the table list is empty, we first obtain
  * a list of tables from the database.
+ *
+ * If 'password' is provided by caller, enforce the password used for
+ * subsequent connections to this value without asking a new value from
+ * prompt.
  */
 static void
 vacuum_one_database(const char *dbname, vacuumingOptions *vacopts,
 					int stage,
 					SimpleStringList *tables,
 					const char *host, const char *port,
-					const char *username, enum trivalue prompt_password,
+					const char *username, const char *password,
+					enum trivalue prompt_password,
 					int concurrentCons,
 					const char *progname, bool echo, bool quiet)
 {
@@ -365,8 +373,8 @@ vacuum_one_database(const char *dbname, vacuumingOptions *vacopts,
 		fflush(stdout);
 	}
 
-	conn = connectDatabase(dbname, host, port, username, prompt_password,
-						   progname, false);
+	conn = connectDatabase(dbname, host, port, username, password,
+						   prompt_password, progname, false);
 
 	initPQExpBuffer(&sql);
 
@@ -424,10 +432,15 @@ vacuum_one_database(const char *dbname, vacuumingOptions *vacopts,
 	init_slot(slots, conn);
 	if (parallel)
 	{
+		const char *password_user = NULL;
+
+		if (PQconnectionUsedPassword(conn))
+			password_user = PQpass(conn);
+
 		for (i = 1; i < concurrentCons; i++)
 		{
-			conn = connectDatabase(dbname, host, port, username, prompt_password,
-								   progname, false);
+			conn = connectDatabase(dbname, host, port, username, password_user,
+								   prompt_password, progname, false);
 			init_slot(slots + i, conn);
 		}
 	}
@@ -542,12 +555,19 @@ vacuum_all_databases(vacuumingOptions *vacopts,
 	PGresult   *result;
 	int			stage;
 	int			i;
+	char	   *password = NULL;
 
 	conn = connectMaintenanceDatabase(maintenance_db, host, port,
 									  username, prompt_password, progname);
+
 	result = executeQuery(conn,
 			"SELECT datname FROM pg_database WHERE datallowconn ORDER BY 1;",
 						  progname, echo);
+
+	/* Save password already given by user for subsequent operations */
+	if (PQconnectionUsedPassword(conn))
+		password = pg_strdup(PQpass(conn));
+
 	PQfinish(conn);
 
 	if (analyze_in_stages)
@@ -570,7 +590,8 @@ vacuum_all_databases(vacuumingOptions *vacopts,
 				vacuum_one_database(dbname, vacopts,
 									stage,
 									NULL,
-									host, port, username, prompt_password,
+									host, port, username, password,
+									prompt_password,
 									concurrentCons,
 									progname, echo, quiet);
 			}
@@ -586,13 +607,16 @@ vacuum_all_databases(vacuumingOptions *vacopts,
 			vacuum_one_database(dbname, vacopts,
 								ANALYZE_NO_STAGE,
 								NULL,
-								host, port, username, prompt_password,
+								host, port, username, password,
+								prompt_password,
 								concurrentCons,
 								progname, echo, quiet);
 		}
 	}
 
 	PQclear(result);
+	if (password)
+		free(password);
 }
 
 /*
